
#include <time.h>

#include <idsa_internal.h>

#include "set.h"
#include "messages.h"

#define P_MESSUP   0.01		/* fudge factor: the chance that my system breaks */

static IDSA_EVENT *message_setup(STATE_SET * s);
static int message_run(STATE_SET * s, IDSA_EVENT * e);

static IDSA_EVENT *message_setup(STATE_SET * s)
{
  IDSA_EVENT *result;

  result = s->s_internal;

  idsa_event_copy(result, s->s_template);
  idsa_time(result, time(NULL));

  return result;
}

static int message_run(STATE_SET * s, IDSA_EVENT * e)
{
  int result;

  result = set_do(s, e, NULL, NULL);
  set_clear(s);

  return result;
}

int message_start(STATE_SET * s, char *v)
{
  IDSA_EVENT *e;

  e = message_setup(s);

  idsa_event_scan(e, "start", NULL, 0, -1.0, P_MESSUP, P_MESSUP, "version",
		  IDSA_T_STRING, v, NULL);

  return message_run(s, e);
}

int message_stop(STATE_SET * s)
{
  IDSA_EVENT *e;

  e = message_setup(s);
  idsa_event_scan(e, "stop", NULL, 0, 1.0, 0.0, 0.0, NULL);

  return message_run(s, e);
}

int message_connect(STATE_SET * s, pid_t p, uid_t u, gid_t g)
{
  IDSA_EVENT *e;
  double d;

  d = 1.0 / (double) (s->s_jobmax);

  e = message_setup(s);
  idsa_event_scan(e, "connect", NULL, 1, d, P_MESSUP * d, P_MESSUP * d,
		  NULL);

  idsa_event_setappend(e, "client_pid", IDSA_T_PID, &p);
  idsa_event_setappend(e, "client_uid", IDSA_T_UID, &u);
  idsa_event_setappend(e, "client_gid", IDSA_T_GID, &g);

  return message_run(s, e);
}

int message_disconnect(STATE_SET * s, pid_t p, uid_t u, gid_t g)
{
  IDSA_EVENT *e;

  e = message_setup(s);
  idsa_event_scan(e, "disconnect", NULL, 0, 0.0, 0.0, P_MESSUP, NULL);

  idsa_event_setappend(e, "client_pid", IDSA_T_PID, &p);
  idsa_event_setappend(e, "client_uid", IDSA_T_UID, &u);
  idsa_event_setappend(e, "client_gid", IDSA_T_GID, &g);

  return message_run(s, e);
}

int message_error_system(STATE_SET * s, int r, char *n)
{
  IDSA_EVENT *e;

  e = message_setup(s);
  idsa_event_scan(e, "error", NULL, 0, IDSA_R_MINOR, 0.0, 0.0, "error",
		  IDSA_T_STRING, "system", "module", IDSA_T_STRING, n,
		  NULL);

  idsa_event_setappend(e, "errno", IDSA_T_ERRNO, &r);

#ifdef TRACE
  idsa_event_dump(e, stderr);
#endif

  return message_run(s, e);
}

int message_error_unhandled(STATE_SET * s, char *n)
{
  IDSA_EVENT *e;

  e = message_setup(s);
  idsa_event_scan(e, "error", NULL, 0, 0.0, 0.0, 0.0, "error",
		  IDSA_T_STRING, "unhandled", "unhandled", IDSA_T_STRING,
		  n, NULL);

#ifdef TRACE
  idsa_event_dump(e, stderr);
#endif

  return message_run(s, e);
}

int message_error_output(STATE_SET * s, LOG_OUTPUT * o)
{
  IDSA_EVENT *e;

  e = message_setup(s);
  idsa_event_scan(e, "error", NULL, 0, IDSA_R_MINOR, 0.0, IDSA_R_MINOR,
		  "error", IDSA_T_STRING, "output", "target",
		  IDSA_T_STRING, o->o_name, NULL);

  idsa_event_setappend(e, "drop_count", IDSA_T_INT, &(o->o_dropcount));
  idsa_event_setappend(e, "errno", IDSA_T_ERRNO, &(o->o_errno));

  return message_run(s, e);
}
