#ifndef _IDSA_QUEUE_H_
#define _IDSA_QUEUE_H_

#include <time.h>

#include <sys/types.h>

#include <idsa_internal.h>

#include "types.h"

struct queue_item{
  QUEUE_ITEM *i_timenext; /* linked list of timeouts */
  QUEUE_ITEM *i_timeprev;

  QUEUE_ITEM *i_ordernext; /* linked list in order which it is compared */
  QUEUE_ITEM *i_orderprev;

  time_t i_timeout; /* time when it becomes stale */
  IDSA_UNIT *i_unit;
};

struct queue_keep{
  char k_name[IDSA_M_NAME]; /* the name of the field to be kept */
  int k_reserved; /* the index of the field in case it is reserved */

  QUEUE_VAR *k_queue;

  time_t k_timeout;
};

struct queue_match{
  char m_name[IDSA_M_NAME]; /* the name of the field to be matched */
  int m_reserved; /* the index of the field in case it is reserved */

  QUEUE_VAR *m_queue;

  int m_count;
  RULE_MATCH **m_match;
};

struct queue_var{
  unsigned int v_type; /* type of variable */

  char v_var[IDSA_M_NAME]; /* its $variable name */

  int v_size; /* number of items in this variable: if 0 then not setup yet */

  QUEUE_ITEM *v_orderhead;
  QUEUE_ITEM *v_ordertail;

  QUEUE_ITEM *v_timehead;
  QUEUE_ITEM *v_timetail;

  QUEUE_ITEM *v_free;

  QUEUE_VAR *v_next;
};

/* at startup ***************************************************************/

void queue_item_free(QUEUE_ITEM *qi);
void queue_keep_free(QUEUE_KEEP *qk);
void queue_match_free(QUEUE_MATCH *qm);
void queue_var_free(QUEUE_VAR *qv);

RULE_MATCH *queue_match_make(STATE_SET *s, char *v, unsigned int t, char *n);
QUEUE_KEEP *queue_keep_make(STATE_SET *s, char *v, unsigned int t, char *n);

void queue_set_size(STATE_SET *s, QUEUE_KEEP *qk, int z);
void queue_set_timeout(STATE_SET *s, QUEUE_KEEP *qk, int t);

/* run time *****************************************************************/

int queue_trigger(QUEUE_MATCH *qm, IDSA_EVENT *e);
int queue_run(QUEUE_KEEP *qk, IDSA_EVENT *e);

#include <stdio.h>

void queue_dump(QUEUE_VAR *qv, FILE *f);

#endif
